@@ -396,6 +396,17 @@ def _check_is_analyzed(table):
                              .format(table.__class__.__name__))
 
 
+def _bin_str_label_format(bin_str, max_length=27):
+    _bin_str = []
+    for bs in bin_str:
+        label = str(bs)
+        if len(label) > max_length:
+            label = label[:max_length] + '...'
+        _bin_str.append(label)
+
+    return _bin_str
+
+
 class BinningTable:
     """Binning table to summarize optimal binning of a numerical or categorical
     variable with respect to a binary target.
@@ -472,6 +483,7 @@ class BinningTable:
         self._quality_score = None
         self._ks = None
 
+        self._bin_str = False
         self._is_built = False
         self._is_analyzed = False
 
@@ -563,6 +575,8 @@ class BinningTable:
         else:
             bin_str.extend(["Special", "Missing"])
 
+        self._bin_str = bin_str
+
         df = pd.DataFrame({
             "Bin": bin_str,
             "Count": n_records,
@@ -585,7 +599,7 @@ class BinningTable:
         return df
 
     def plot(self, metric="woe", add_special=True, add_missing=True,
-             style="bin", savefig=None):
+             style="bin", show_bin_labels=False, savefig=None):
         """Plot the binning table.
 
         Visualize the non-event and event count, and the Weight of Evidence or
@@ -608,6 +622,10 @@ class BinningTable:
             style="actual", show the plot with the actual scale, i.e, actual
             bin widths.
 
+        show_bin_labels: bool (default=False)
+            Whether to show the bin label instead of the bin id on the x-axis.
+            For long labels (length > 27), labels are truncated.
+
         savefig : str or None (default=None)
             Path to save the plot figure.
         """
@@ -629,6 +647,14 @@ class BinningTable:
             raise ValueError('Invalid value for style. Allowed string '
                              'values are "bin" and "actual".')
 
+        if not isinstance(show_bin_labels, bool):
+            raise TypeError("show_bin_labels must be a boolean; got {}."
+                            .format(show_bin_labels))
+
+        if show_bin_labels and style == "actual":
+            raise ValueError('show_bin_labels only supported when '
+                             'style="actual".')
+
         if style == "actual":
             # Hide special and missing bin
             add_special = False
@@ -743,6 +769,17 @@ class BinningTable:
             ax2.set_ylabel(metric_label, fontsize=13)
             ax2.xaxis.set_major_locator(mtick.MultipleLocator(1))
 
+            if show_bin_labels:
+                ax1.set_xlabel("Bin", fontsize=12)
+                ax1.set_xticks(np.arange(len(self._bin_str)))
+
+                if self.dtype == "categorical":
+                    bin_str = _bin_str_label_format(self._bin_str)
+                else:
+                    bin_str = self._bin_str
+
+                ax1.set_xticklabels(bin_str, rotation=45, ha="right")
+
         elif style == "actual":
             _n_nonevent = self.n_nonevent[:-(self._n_specials + 1)]
             _n_event = self.n_event[:-(self._n_specials + 1)]
@@ -785,8 +822,14 @@ class BinningTable:
             ax2.set_ylabel(metric_label, fontsize=13)
 
         plt.title(self.name, fontsize=14)
-        plt.legend(handles, labels, loc="upper center",
-                   bbox_to_anchor=(0.5, -0.2), ncol=2, fontsize=12)
+
+        if show_bin_labels:
+            legend_high = max(map(len, bin_str)) / 70 + 0.2
+            plt.legend(handles, labels, loc="upper center",
+                       bbox_to_anchor=(0.5, -legend_high), ncol=2, fontsize=12)
+        else:
+            plt.legend(handles, labels, loc="upper center",
+                       bbox_to_anchor=(0.5, -0.2), ncol=2, fontsize=12)
 
         if savefig is None:
             plt.show()
@@ -1060,7 +1103,9 @@ class MulticlassBinningTable:
         self._n_specials = None
         self._quality_score = None
 
+        self._bin_str = False
         self._is_built = False
+        self._is_analyzed = False
 
     def build(self, show_digits=2, add_totals=True):
         """Build the binning table.
@@ -1118,6 +1163,8 @@ class MulticlassBinningTable:
         else:
             bin_str.extend(["Special", "Missing"])
 
+        self._bin_str = bin_str
+
         dict_event = {"Event_{0}".format(cl): n_event[:, i]
                       for i, cl in enumerate(self.classes)}
 
@@ -1142,7 +1189,8 @@ class MulticlassBinningTable:
 
         return df
 
-    def plot(self, add_special=True, add_missing=True, savefig=None):
+    def plot(self, add_special=True, add_missing=True, show_bin_labels=False,
+             savefig=None):
         """Plot the binning table.
 
         Visualize event count and event rate values for each class.
@@ -1155,11 +1203,27 @@ class MulticlassBinningTable:
         add_missing : bool (default=True)
             Whether to add the special values bin.
 
+        show_bin_labels: bool (default=False)
+            Whether to show the bin label instead of the bin id on the x-axis.
+            For long labels (length > 27), labels are truncated.
+
         savefig : str or None (default=None)
             Path to save the plot figure.
         """
         _check_is_built(self)
 
+        if not isinstance(add_special, bool):
+            raise TypeError("add_special must be a boolean; got {}."
+                            .format(add_special))
+
+        if not isinstance(add_missing, bool):
+            raise TypeError("add_missing must be a boolean; got {}."
+                            .format(add_missing))
+
+        if not isinstance(show_bin_labels, bool):
+            raise TypeError("show_bin_labels must be a boolean; got {}."
+                            .format(show_bin_labels))
+
         n_bins = len(self._n_records)
         n_metric = n_bins - 1 - self._n_specials
         n_classes = len(self.classes)
@@ -1255,9 +1319,20 @@ class MulticlassBinningTable:
         ax2.set_ylabel(metric_label, fontsize=13)
         ax2.xaxis.set_major_locator(mtick.MultipleLocator(1))
 
+        if show_bin_labels:
+            ax1.set_xlabel("Bin", fontsize=12)
+            ax1.set_xticks(np.arange(len(self._bin_str)))
+            ax1.set_xticklabels(self._bin_str, rotation=45, ha="right")
+
         plt.title(self.name, fontsize=14)
-        plt.legend(handles, labels, loc="upper center",
-                   bbox_to_anchor=(0.5, -0.2), ncol=2, fontsize=12)
+
+        if show_bin_labels:
+            legend_high = max(map(len, self._bin_str)) / 70 + 0.2
+            plt.legend(handles, labels, loc="upper center",
+                       bbox_to_anchor=(0.5, -legend_high), ncol=2, fontsize=12)
+        else:
+            plt.legend(handles, labels, loc="upper center",
+                       bbox_to_anchor=(0.5, -0.2), ncol=2, fontsize=12)
 
         if savefig is None:
             plt.show()
@@ -1469,11 +1544,15 @@ class ContinuousBinningTable:
 
         self._mean = None
         self._iv = None
+        self._woe = None
+        self._t_mean = None
         self._hhi = None
         self._hhi_norm = None
         self._n_specials = None
 
+        self._bin_str = None
         self._is_built = False
+        self._is_analyzed = False
 
     def build(self, show_digits=2, add_totals=True):
         """
@@ -1501,15 +1580,16 @@ class ContinuousBinningTable:
         mask = (self.n_records > 0)
         self._mean = np.zeros(len(self.n_records))
         self._mean[mask] = self.sums[mask] / self.n_records[mask]
-        if self.n_records[-1] > 0:
-            self._mean[-1] = 0
 
         # Compute divergence measure (continuous adaptation)
         woe = self._mean - t_mean
         iv = np.absolute(woe) * p_records
         t_iv = iv.sum()
+        t_woe = np.absolute(woe).sum()
 
         self._iv = t_iv
+        self._woe = t_woe
+        self._t_mean = t_mean
 
         # Compute HHI
         self._hhi = hhi(p_records)
@@ -1533,6 +1613,8 @@ class ContinuousBinningTable:
         else:
             bin_str.extend(["Special", "Missing"])
 
+        self._bin_str = bin_str
+
         df = pd.DataFrame({
             "Bin": bin_str,
             "Count": self.n_records,
@@ -1552,7 +1634,7 @@ class ContinuousBinningTable:
             t_max = np.nanmax(self.max_target)
             t_n_zeros = self.n_zeros.sum()
             totals = ["", t_n_records, 1, t_sum, "", t_mean, t_min, t_max,
-                      t_n_zeros, "", t_iv]
+                      t_n_zeros, t_woe, t_iv]
             df.loc["Totals"] = totals
 
         self._is_built = True
@@ -1560,7 +1642,7 @@ class ContinuousBinningTable:
         return df
 
     def plot(self, add_special=True, add_missing=True, style="bin",
-             savefig=None):
+             show_bin_labels=False, savefig=None):
         """Plot the binning table.
 
         Visualize records count and mean values.
@@ -1578,6 +1660,10 @@ class ContinuousBinningTable:
             style="actual", show the plot with the actual scale, i.e, actual
             bin widths.
 
+        show_bin_labels: bool (default=False)
+            Whether to show the bin label instead of the bin id on the x-axis.
+            For long labels (length > 27), labels are truncated.
+
         savefig : str or None (default=None)
             Path to save the plot figure.
         """
@@ -1595,6 +1681,14 @@ class ContinuousBinningTable:
             raise ValueError('Invalid value for style. Allowed string '
                              'values are "bin" and "actual".')
 
+        if not isinstance(show_bin_labels, bool):
+            raise TypeError("show_bin_labels must be a boolean; got {}."
+                            .format(show_bin_labels))
+
+        if show_bin_labels and style == "actual":
+            raise ValueError('show_bin_labels only supported when '
+                             'style="actual".')
+
         if style == "actual":
             # Hide special and missing bin
             add_special = False
@@ -1697,6 +1791,17 @@ class ContinuousBinningTable:
             ax2.set_ylabel(metric_label, fontsize=13)
             ax2.xaxis.set_major_locator(mtick.MultipleLocator(1))
 
+            if show_bin_labels:
+                ax1.set_xlabel("Bin", fontsize=12)
+                ax1.set_xticks(np.arange(len(self._bin_str)))
+
+                if self.dtype == "categorical":
+                    bin_str = _bin_str_label_format(self._bin_str)
+                else:
+                    bin_str = self._bin_str
+
+                ax1.set_xticklabels(bin_str, rotation=45, ha="right")
+
         elif style == "actual":
             _n_records = self.n_records[:-(self._n_specials + 1)]
 
@@ -1737,8 +1842,14 @@ class ContinuousBinningTable:
             ax2.set_ylabel(metric_label, fontsize=13)
 
         plt.title(self.name, fontsize=14)
-        plt.legend(handles, labels, loc="upper center",
-                   bbox_to_anchor=(0.5, -0.2), ncol=3, fontsize=12)
+
+        if show_bin_labels:
+            legend_high = max(map(len, bin_str)) / 70 + 0.2
+            plt.legend(handles, labels, loc="upper center",
+                       bbox_to_anchor=(0.5, -legend_high), ncol=2, fontsize=12)
+        else:
+            plt.legend(handles, labels, loc="upper center",
+                       bbox_to_anchor=(0.5, -0.2), ncol=2, fontsize=12)
 
         if savefig is None:
             plt.show()
@@ -1813,8 +1924,13 @@ class ContinuousBinningTable:
             df_tests_string = " " * tab + "None"
 
         # Quality score
+        if self._t_mean == 0:
+            rwoe = self._woe
+        else:
+            rwoe = self._woe / abs(self._t_mean)
+
         self._quality_score = continuous_binning_quality_score(
-            p_values, self._hhi_norm)
+            rwoe, p_values, self._hhi_norm)
 
         # Monotonic trend
         type_mono = type_of_monotonic_trend(self._mean[:-2])
@@ -1827,6 +1943,8 @@ class ContinuousBinningTable:
             "  General metrics"
             "\n\n"
             "    IV                  {:>15.8f}\n"
+            "    WoE                 {:>15.8f}\n"
+            "    WoE (normalized)    {:>15.8f}\n"
             "    HHI                 {:>15.8f}\n"
             "    HHI (normalized)    {:>15.8f}\n"
             "    Quality score       {:>15.8f}\n"
@@ -1834,12 +1952,14 @@ class ContinuousBinningTable:
             "  Monotonic trend       {:>15}\n"
             "\n"
             "  Significance tests\n\n{}\n"
-            ).format(self._iv, self._hhi, self._hhi_norm, self._quality_score,
-                     type_mono, df_tests_string)
+            ).format(self._iv, self._woe, rwoe, self._hhi, self._hhi_norm,
+                     self._quality_score, type_mono, df_tests_string)
 
         if print_output:
             print(report)
 
+        self._is_analyzed = True
+
     @property
     def iv(self):
         """The Information Value (IV).
@@ -1854,6 +1974,27 @@ class ContinuousBinningTable:
 
         return self._iv
 
+    @property
+    def woe(self):
+        r"""The sum of absolute WoEs.
+
+        This metric is computed as follows:
+
+        .. math::
+
+            WoE = \sum_{i=1}^n |U_i - \mu|,
+
+        where :math:`U_i` is the target mean value for each bin, :math:`\mu` is
+        the total target mean.
+
+        Returns
+        -------
+        woe : float
+        """
+        _check_is_built(self)
+
+        return self._woe
+
     @property
     def quality_score(self):
         """The quality score (QS).
