@@ -15,14 +15,13 @@ from ...options import optimal_binning_sketch_options
 
 
 def print_timing(solver_type, solver, time_total, time_prebinning, time_solver,
-                 time_postprocessing):
+                 time_optimizer, time_postprocessing):
 
     p_prebinning = time_prebinning / time_total
     p_solver = time_solver / time_total
     p_postprocessing = time_postprocessing / time_total
 
     if solver_type == "cp" and solver is not None:
-        time_optimizer = solver.WallTime()
         time_model_generation = time_solver - time_optimizer
         p_model_generation = time_model_generation / time_solver
         p_optimizer = time_optimizer / time_solver
@@ -77,9 +76,9 @@ def print_streaming_timing(memory_usage, n_records, n_add, time_add, n_solve,
 
 def print_binning_information(binning_type, print_level, name, status,
                               solver_type, solver, time_total, time_prebinning,
-                              time_solver, time_postprocessing, n_prebins,
-                              n_refinements, n_records, n_add, time_add,
-                              n_solve, time_solve, memory_usage,
+                              time_solver, time_optimizer, time_postprocessing,
+                              n_prebins, n_refinements, n_records, n_add,
+                              time_add, n_solve, time_solve, memory_usage,
                               dict_user_options):
 
     print_header()
@@ -102,7 +101,7 @@ def print_binning_information(binning_type, print_level, name, status,
                 print_solver_statistics(solver_type, solver)
 
             print_timing(solver_type, solver, time_total, time_prebinning,
-                         time_solver, time_postprocessing)
+                         time_solver, time_optimizer, time_postprocessing)
 
         print_streaming_timing(memory_usage, n_records, n_add, time_add,
                                n_solve, time_solve)
