@@ -215,14 +215,16 @@ def continuous_model_data(n_records, sums, ssums, max_pvalue,
     R = []
     V = []
 
+    t_mean = sums.sum() / n_records.sum()
+
     for i in range(1, n + 1):
         s_n_records = n_records[:i][::-1].cumsum()[::-1]
         s_sums = sums[:i][::-1].cumsum()[::-1]
         s_ssums = ssums[:i][::-1].cumsum()[::-1]
 
         mean = s_sums / s_n_records
-        std = np.sqrt(s_ssums / s_n_records - (s_sums / s_n_records) ** 2)
-        norm = np.absolute(sums[i-1] - s_sums)
+        std = np.sqrt(s_ssums / s_n_records - mean ** 2)
+        norm = np.absolute(mean - t_mean)
 
         if scale is not None:
             mean_scaled = mean * scale
