@@ -24,6 +24,9 @@ from .binning_information import print_binning_information
 from .binning_information import retrieve_status
 
 
+logger = Logger(__name__).logger
+
+
 def _check_parameters(name, estimator, objective, degree, continuous,
                       prebinning_method, max_n_prebins, min_prebin_size,
                       min_n_bins, max_n_bins, min_bin_size, max_bin_size,
@@ -158,8 +161,9 @@ def _check_parameters(name, estimator, objective, degree, continuous,
                                                    len(user_splits_fixed)))
 
     if special_codes is not None:
-        if not isinstance(special_codes, (np.ndarray, list)):
-            raise TypeError("special_codes must be a list or numpy.ndarray.")
+        if not isinstance(special_codes, (np.ndarray, list, dict)):
+            raise TypeError("special_codes must be a dict, list or "
+                            "numpy.ndarray.")
 
     if split_digits is not None:
         if (not isinstance(split_digits, numbers.Integral) or
@@ -269,10 +273,6 @@ class BasePWBinning(Base, BaseEstimator):
         self._time_solver = None
         self._time_postprocessing = None
 
-        # logger
-        self._class_logger = Logger(__name__)
-        self._logger = self._class_logger.logger
-
         self._is_fitted = False
 
     def fit(self, x, y, lb=None, ub=None, check_input=False):
@@ -341,7 +341,7 @@ class BasePWBinning(Base, BaseEstimator):
 
     def _fit_binning(self, x, y, prediction, lb, ub):
         if self.verbose:
-            self._logger.info("Pre-binning: optimal binning started.")
+            logger.info("Pre-binning: optimal binning started.")
 
         time_prebinning = time.perf_counter()
 
@@ -394,8 +394,8 @@ class BasePWBinning(Base, BaseEstimator):
         n_splits = len(splits)
 
         if self.verbose:
-            self._logger.info("Pre-binning: number of splits: {}."
-                              .format(n_splits))
+            logger.info("Pre-binning: number of splits: {}."
+                        .format(n_splits))
 
         # Prepare optimization model data
         n_bins = n_splits + 1
@@ -406,7 +406,7 @@ class BasePWBinning(Base, BaseEstimator):
             pred_subsamples = prediction
 
             if self.verbose:
-                self._logger.info("Pre-binning: no need for subsamples.")
+                logger.info("Pre-binning: no need for subsamples.")
         else:
             indices = np.digitize(x, splits, right=False)
             [_, x_subsamples, _, pred_subsamples,
@@ -415,18 +415,18 @@ class BasePWBinning(Base, BaseEstimator):
                 random_state=self.random_state)
 
             if self.verbose:
-                self._logger.info("Pre-binning: number of subsamples: {}."
-                                  .format(self.n_subsamples))
+                logger.info("Pre-binning: number of subsamples: {}."
+                            .format(self.n_subsamples))
 
         self._time_prebinning = time.perf_counter() - time_prebinning
 
         if self.verbose:
-            self._logger.info("Pre-binning: optimal binning terminated. "
-                              "Time {:.4}s.".format(self._time_prebinning))
+            logger.info("Pre-binning: optimal binning terminated. Time {:.4}s."
+                        .format(self._time_prebinning))
 
         # LP problem
         if self.verbose:
-            self._logger.info("Optimizer started.")
+            logger.info("Optimizer started.")
 
         if self.monotonic_trend == "auto":
             indices = np.digitize(x, splits, right=False)
@@ -441,8 +441,8 @@ class BasePWBinning(Base, BaseEstimator):
                 monotonic = "valley"
 
             if self.verbose:
-                self._logger.info("Optimizer: {} monotonic trend."
-                                  .format(monotonic))
+                logger.info("Optimizer: {} monotonic trend."
+                            .format(monotonic))
         else:
             monotonic = self.monotonic_trend
 
@@ -464,8 +464,8 @@ class BasePWBinning(Base, BaseEstimator):
         self._time_solver = time.perf_counter() - time_solver
 
         if self.verbose:
-            self._logger.info("Optimizer terminated. Time: {:.4f}s"
-                              .format(self._time_solver))
+            logger.info("Optimizer terminated. Time: {:.4f}s"
+                        .format(self._time_solver))
 
     @property
     def binning_table(self):
