@@ -9,6 +9,8 @@ import time
 
 import numpy as np
 
+from ...binning.binning_statistics import target_info_special_continuous
+from ...logging import Logger
 from .base import _check_parameters
 from .base import BasePWBinning
 from .binning_statistics import PWContinuousBinningTable
@@ -16,6 +18,9 @@ from .metrics import continuous_metrics
 from .transformations import transform_continuous_target
 
 
+logger = Logger(__name__).logger
+
+
 class ContinuousOptimalPWBinning(BasePWBinning):
     """Optimal Piecewise binning of a numerical variable with respect to a
     binary target.
@@ -87,7 +92,7 @@ class ContinuousOptimalPWBinning(BasePWBinning):
         Number of subsamples to fit the piecewise regression algorithm. If
         None, all values are considered.
 
-    max_pvalue : float or None, optional (default=0.05)
+    max_pvalue : float or None, optional (default=None)
         The maximum p-value among bins. The Z-test is used to detect bins
         not satisfying the p-value constraint. Option supported by solvers
         "cp" and "mip".
@@ -112,7 +117,7 @@ class ContinuousOptimalPWBinning(BasePWBinning):
     user_splits_fixed : array-like or None (default=None)
         The list of pre-binning split points that must be fixed.
 
-    special_codes : array-like or None, optional (default=None)
+    special_codes : array-like, dict or None, optional (default=None)
         List of special codes. Use special codes to specify the data values
         that must be treated separately.
 
@@ -276,21 +281,21 @@ class ContinuousOptimalPWBinning(BasePWBinning):
         time_init = time.perf_counter()
 
         if self.verbose:
-            self._logger.info("Optimal piecewise binning started.")
-            self._logger.info("Options: check parameters.")
+            logger.info("Optimal piecewise binning started.")
+            logger.info("Options: check parameters.")
 
         _check_parameters(**self.get_params(deep=False), estimator=None,
                           problem_type=self._problem_type)
 
         # Pre-processing
         if self.verbose:
-            self._logger.info("Pre-processing started.")
+            logger.info("Pre-processing started.")
 
         self._n_samples = len(x)
 
         if self.verbose:
-            self._logger.info("Pre-processing: number of samples: {}"
-                              .format(self._n_samples))
+            logger.info("Pre-processing: number of samples: {}"
+                        .format(self._n_samples))
 
         time_preprocessing = time.perf_counter()
 
@@ -304,22 +309,22 @@ class ContinuousOptimalPWBinning(BasePWBinning):
             n_missing = len(x_missing)
             n_special = len(x_special)
 
-            self._logger.info("Pre-processing: number of clean samples: {}"
-                              .format(n_clean))
+            logger.info("Pre-processing: number of clean samples: {}"
+                        .format(n_clean))
 
-            self._logger.info("Pre-processing: number of missing samples: {}"
-                              .format(n_missing))
+            logger.info("Pre-processing: number of missing samples: {}"
+                        .format(n_missing))
 
-            self._logger.info("Pre-processing: number of special samples: {}"
-                              .format(n_special))
+            logger.info("Pre-processing: number of special samples: {}"
+                        .format(n_special))
 
             if self.outlier_detector is not None:
                 n_outlier = self._n_samples-(n_clean + n_missing + n_special)
-                self._logger.info("Pre-processing: number of outlier samples: "
-                                  "{}".format(n_outlier))
+                logger.info("Pre-processing: number of outlier samples: {}"
+                            .format(n_outlier))
 
-            self._logger.info("Pre-processing terminated. Time: {:.4f}s"
-                              .format(self._time_preprocessing))
+            logger.info("Pre-processing terminated. Time: {:.4f}s"
+                        .format(self._time_preprocessing))
 
         # Pre-binning
         self._time_estimator = 0
@@ -330,19 +335,24 @@ class ContinuousOptimalPWBinning(BasePWBinning):
 
         # Post-processing
         if self.verbose:
-            self._logger.info("Post-processing started.")
-            self._logger.info("Post-processing: compute binning information.")
+            logger.info("Post-processing started.")
+            logger.info("Post-processing: compute binning information.")
 
         time_postprocessing = time.perf_counter()
 
         # Compute n_records and sum for special and missing
-        self._n_records_special = len(y_special)
-        self._sum_special = np.sum(y_special)
-        self._n_zeros_special = np.count_nonzero(y_special == 0)
-        if len(y_special):
-            self._std_special = np.std(y_special)
-            self._min_target_special = np.min(y_special)
-            self._max_target_special = np.max(y_special)
+        # self._n_records_special = len(y_special)
+        # self._sum_special = np.sum(y_special)
+        # self._n_zeros_special = np.count_nonzero(y_special == 0)
+        # if len(y_special):
+        #     self._std_special = np.std(y_special)
+        #     self._min_target_special = np.min(y_special)
+        #     self._max_target_special = np.max(y_special)
+
+        [self._n_records_special, self._sum_special, self._n_zeros_special,
+         self._std_special, self._min_target_special,
+         self._max_target_special] = target_info_special_continuous(
+            self.special_codes, x_special, y_special)
 
         self._n_records_missing = len(y_missing)
         self._sum_missing = np.sum(y_missing)
@@ -353,29 +363,30 @@ class ContinuousOptimalPWBinning(BasePWBinning):
             self._max_target_missing = np.max(y_missing)
 
         bt = self._optb.binning_table.build(add_totals=False)
-        n_records = bt["Count"].values
-        sums = bt["Sum"].values
-        stds = bt["Std"].values
-        min_target = bt["Min"].values
-        max_target = bt["Max"].values
-        n_zeros = bt["Zeros count"].values
+        n_records = bt["Count"].values[:-2]
+        sums = bt["Sum"].values[:-2]
+        stds = bt["Std"].values[:-2]
+        min_target = bt["Min"].values[:-2]
+        max_target = bt["Max"].values[:-2]
+        n_zeros = bt["Zeros count"].values[:-2]
 
-        n_records[self._n_bins] = self._n_records_special
-        n_records[self._n_bins + 1] = self._n_records_missing
-        sums[self._n_bins] = self._sum_special
-        sums[self._n_bins + 1] = self._sum_missing
-        stds[self._n_bins] = self._std_special
-        stds[self._n_bins + 1] = self._std_missing
-        min_target[self._n_bins] = self._min_target_special
-        min_target[self._n_bins + 1] = self._min_target_missing
-        max_target[self._n_bins] = self._max_target_special
-        max_target[self._n_bins + 1] = self._max_target_missing
-        n_zeros[self._n_bins] = self._n_zeros_special
-        n_zeros[self._n_bins + 1] = self._n_zeros_missing
+        n_records = np.r_[n_records, self._n_records_special]
+        sums = np.r_[sums, self._sum_special]
+        stds = np.r_[stds, self._std_special]
+        min_target = np.r_[min_target, self._min_target_special]
+        max_target = np.r_[max_target, self._max_target_special]
+        n_zeros = np.r_[n_zeros, self._n_zeros_special]
+
+        n_records = np.r_[n_records, self._n_records_missing]
+        sums = np.r_[sums, self._sum_missing]
+        stds = np.r_[stds, self._std_missing]
+        min_target = np.r_[min_target, self._min_target_missing]
+        max_target = np.r_[max_target, self._max_target_missing]
+        n_zeros = np.r_[n_zeros, self._n_zeros_missing]
 
         # Compute metrics
         if self.verbose:
-            self._logger.info("Post-processing: compute performance metrics.")
+            logger.info("Post-processing: compute performance metrics.")
 
         d_metrics = continuous_metrics(
             x_clean, y_clean, self._optb.splits, self._c, lb, ub,
@@ -384,25 +395,23 @@ class ContinuousOptimalPWBinning(BasePWBinning):
 
         # Binning table
         self._binning_table = PWContinuousBinningTable(
-            self.name, self._optb.splits, self._c, n_records, sums, stds,
-            min_target, max_target, n_zeros, lb, ub, x_clean.min(),
-            x_clean.max(), d_metrics)
+            self.name, self.special_codes, self._optb.splits, self._c,
+            n_records, sums, stds, min_target, max_target, n_zeros, lb, ub,
+            x_clean.min(), x_clean.max(), d_metrics)
 
         self._time_postprocessing = time.perf_counter() - time_postprocessing
 
         if self.verbose:
-            self._logger.info("Post-processing terminated. Time: {:.4f}s"
-                              .format(self._time_postprocessing))
+            logger.info("Post-processing terminated. Time: {:.4f}s"
+                        .format(self._time_postprocessing))
 
         self._time_total = time.perf_counter() - time_init
 
         if self.verbose:
-            self._logger.info("Optimal piecewise binning terminated. "
-                              "Status: {}. Time: {:.4f}s"
-                              .format(self._status, self._time_total))
+            logger.info("Optimal piecewise binning terminated. Status: {}. "
+                        "Time: {:.4f}s".format(self._status, self._time_total))
 
         # Completed successfully
-        self._class_logger.close()
         self._is_fitted = True
 
         return self
