@@ -13,6 +13,7 @@ import numpy as np
 
 from sklearn.utils import check_array
 
+from ...information import solver_statistics
 from ...logging import Logger
 from ...binning.preprocessing import split_data_scenarios
 from ..binning import OptimalBinning
@@ -23,6 +24,9 @@ from ..cp import BinningCP
 from ..prebinning import PreBinning
 
 
+logger = Logger(__name__).logger
+
+
 def _check_parameters(name, prebinning_method, max_n_prebins, min_prebin_size,
                       min_n_bins, max_n_bins, min_bin_size, max_bin_size,
                       monotonic_trend, min_event_rate_diff, max_pvalue,
@@ -220,7 +224,7 @@ class SBOptimalBinning(OptimalBinning):
         option currently only applies when ``monotonic_trend`` is "ascending",
         "descending", "peak_heuristic" or "valley_heuristic".
 
-    max_pvalue : float or None, optional (default=0.05)
+    max_pvalue : float or None, optional (default=None)
         The maximum p-value among bins. The Z-test is used to detect bins
         not satisfying the p-value constraint.
 
@@ -325,12 +329,9 @@ class SBOptimalBinning(OptimalBinning):
         self._time_preprocessing = None
         self._time_prebinning = None
         self._time_solver = None
+        self._time_optimizer = None
         self._time_postprocessing = None
 
-        # logger
-        self._class_logger = Logger(__name__)
-        self._logger = self._class_logger.logger
-
         self._is_fitted = False
 
     def fit(self, X, Y, weights=None, check_input=False):
@@ -422,14 +423,14 @@ class SBOptimalBinning(OptimalBinning):
         self._n_scenarios = len(X)
 
         if self.verbose:
-            self._logger.info("Optimal binning started.")
-            self._logger.info("Options: check parameters.")
+            logger.info("Optimal binning started.")
+            logger.info("Options: check parameters.")
 
         _check_parameters(**self.get_params())
 
         # Pre-processing
         if self.verbose:
-            self._logger.info("Pre-processing started.")
+            logger.info("Pre-processing started.")
 
         time_preprocessing = time.perf_counter()
 
@@ -437,8 +438,8 @@ class SBOptimalBinning(OptimalBinning):
         self._n_samples = sum(self._n_samples_scenario)
 
         if self.verbose:
-            self._logger.info("Pre-processing: number of samples: {}"
-                              .format(self._n_samples))
+            logger.info("Pre-processing: number of samples: {}"
+                        .format(self._n_samples))
 
         [x_clean, y_clean, x_missing, y_missing, x_special, y_special,
          w] = split_data_scenarios(X, Y, weights, self.special_codes,
@@ -451,21 +452,21 @@ class SBOptimalBinning(OptimalBinning):
             n_missing = len(x_missing)
             n_special = len(x_special)
 
-            self._logger.info("Pre-processing: number of clean samples: {}"
-                              .format(n_clean))
+            logger.info("Pre-processing: number of clean samples: {}"
+                        .format(n_clean))
 
-            self._logger.info("Pre-processing: number of missing samples: {}"
-                              .format(n_missing))
+            logger.info("Pre-processing: number of missing samples: {}"
+                        .format(n_missing))
 
-            self._logger.info("Pre-processing: number of special samples: {}"
-                              .format(n_special))
+            logger.info("Pre-processing: number of special samples: {}"
+                        .format(n_special))
 
-            self._logger.info("Pre-processing terminated. Time: {:.4f}s"
-                              .format(self._time_preprocessing))
+            logger.info("Pre-processing terminated. Time: {:.4f}s"
+                        .format(self._time_preprocessing))
 
         # Pre-binning
         if self.verbose:
-            self._logger.info("Pre-binning started.")
+            logger.info("Pre-binning started.")
 
         time_prebinning = time.perf_counter()
 
@@ -495,21 +496,21 @@ class SBOptimalBinning(OptimalBinning):
         self._time_prebinning = time.perf_counter() - time_prebinning
 
         if self.verbose:
-            self._logger.info("Pre-binning: number of prebins: {}"
-                              .format(self._n_prebins))
-            self._logger.info("Pre-binning: number of refinements: {}"
-                              .format(self._n_refinements))
+            logger.info("Pre-binning: number of prebins: {}"
+                        .format(self._n_prebins))
+            logger.info("Pre-binning: number of refinements: {}"
+                        .format(self._n_refinements))
 
-            self._logger.info("Pre-binning terminated. Time: {:.4f}s"
-                              .format(self._time_prebinning))
+            logger.info("Pre-binning terminated. Time: {:.4f}s"
+                        .format(self._time_prebinning))
 
         # Optimization
         self._fit_optimizer(splits, n_nonevent, n_event, weights)
 
         # Post-processing
         if self.verbose:
-            self._logger.info("Post-processing started.")
-            self._logger.info("Post-processing: compute binning information.")
+            logger.info("Post-processing started.")
+            logger.info("Post-processing: compute binning information.")
 
         time_postprocessing = time.perf_counter()
 
@@ -517,7 +518,19 @@ class SBOptimalBinning(OptimalBinning):
         self._n_event = 0
         self._binning_tables = []
 
+        min_x = np.inf
+        max_x = -np.inf
+
         for s in range(self._n_scenarios):
+            min_xs = x_clean[s].min()
+            max_xs = x_clean[s].max()
+
+            if min_xs < min_x:
+                min_x = min_xs
+
+            if max_xs > max_x:
+                max_x = max_xs
+
             s_n_nonevent, s_n_event = bin_info(
                 self._solution, n_nonevent[:, s], n_event[:, s],
                 self._n_nonevent_missing[s], self._n_event_missing[s],
@@ -528,30 +541,30 @@ class SBOptimalBinning(OptimalBinning):
             self._n_event += s_n_event
 
             binning_table = BinningTable(
-                self.name, self.dtype, self._splits_optimal, s_n_nonevent,
-                s_n_event, None, None, None, None, self.user_splits)
+                self.name, self.dtype, self.special_codes,
+                self._splits_optimal, s_n_nonevent, s_n_event, min_xs, max_xs,
+                None, None, self.user_splits)
 
             self._binning_tables.append(binning_table)
 
         self._binning_table = BinningTable(
-            self.name, self.dtype, self._splits_optimal, self._n_nonevent,
-            self._n_event, None, None, None, None, self.user_splits)
+            self.name, self.dtype, self.special_codes, self._splits_optimal,
+            self._n_nonevent, self._n_event, min_x, max_x, None, None,
+            self.user_splits)
 
         self._time_postprocessing = time.perf_counter() - time_postprocessing
 
         if self.verbose:
-            self._logger.info("Post-processing terminated. Time: {:.4f}s"
-                              .format(self._time_postprocessing))
+            logger.info("Post-processing terminated. Time: {:.4f}s"
+                        .format(self._time_postprocessing))
 
         self._time_total = time.perf_counter() - time_init
 
         if self.verbose:
-            self._logger.info("Optimal binning terminated. Status: {}. "
-                              "Time: {:.4f}s"
-                              .format(self._status, self._time_total))
+            logger.info("Optimal binning terminated. Status: {}. Time: {:.4f}s"
+                        .format(self._status, self._time_total))
 
         # Completed successfully
-        self._class_logger.close()
         self._is_fitted = True
 
         return self
@@ -653,8 +666,8 @@ class SBOptimalBinning(OptimalBinning):
             splits = splits_prebinning[~mask_splits]
 
             if self.verbose:
-                self._logger.info("Pre-binning: number prebins removed: {}"
-                                  .format(np.count_nonzero(mask_remove)))
+                logger.info("Pre-binning: number prebins removed: {}"
+                            .format(np.count_nonzero(mask_remove)))
 
             [splits_prebinning, n_nonevent, n_event] = self._compute_prebins(
                 splits, x, y)
@@ -670,10 +683,10 @@ class SBOptimalBinning(OptimalBinning):
             self._solution = np.zeros(len(splits)).astype(bool)
 
             if self.verbose:
-                self._logger.warning("Optimizer: no bins after pre-binning.")
-                self._logger.warning("Optimizer: solver not run.")
+                logger.warning("Optimizer: no bins after pre-binning.")
+                logger.warning("Optimizer: solver not run.")
 
-                self._logger.info("Optimizer terminated. Time: 0s")
+                logger.info("Optimizer terminated. Time: 0s")
             return
 
         if self.min_bin_size is not None:
@@ -699,18 +712,19 @@ class SBOptimalBinning(OptimalBinning):
             weights = np.ones(self._n_scenarios, int)
 
         if self.verbose:
-            self._logger.info("Optimizer: build model...")
+            logger.info("Optimizer: build model...")
 
         optimizer.build_model_scenarios(n_nonevent, n_event, weights)
 
         status, solution = optimizer.solve()
 
         if self.verbose:
-            self._logger.info("Optimizer: solve...")
+            logger.info("Optimizer: solve...")
 
         self._solution = solution
 
-        self._optimizer = optimizer
+        self._optimizer, self._time_optimizer = solver_statistics(
+            self.solver, optimizer.solver_)
         self._status = status
 
         self._splits_optimal = splits[solution[:-1]]
@@ -718,8 +732,8 @@ class SBOptimalBinning(OptimalBinning):
         self._time_solver = time.perf_counter() - time_init
 
         if self.verbose:
-            self._logger.info("Optimizer terminated. Time: {:.4f}s"
-                              .format(self._time_solver))
+            logger.info("Optimizer terminated. Time: {:.4f}s"
+                        .format(self._time_solver))
 
     def binning_table_scenario(self, scenario_id):
         """Return the instantiated binning table corresponding to
