@@ -7,9 +7,9 @@ Scorecard development.
 # Copyright (C) 2020
 
 import numbers
+import pickle
 import time
 
-import dill
 import numpy as np
 import pandas as pd
 
@@ -61,22 +61,26 @@ def _check_parameters(binning_process, estimator, scaling_method,
     if not isinstance(rounding, bool):
         raise TypeError("rounding must be a boolean; got {}.".format(rounding))
 
+    if rounding and scaling_method is None:
+        raise ValueError("rounding is only applied if scaling method is "
+                         "not None.")
+
     if not isinstance(verbose, bool):
         raise TypeError("verbose must be a boolean; got {}.".format(verbose))
 
 
 def _check_scorecard_scaling(scaling_method, scaling_method_params,
-                             target_type):
+                             rounding, target_type):
     if scaling_method is not None:
         if scaling_method == "pdo_odds":
-            default_keys = ["pdo", "odds", "scorecard_points"]
+            default_keys = ("pdo", "odds", "scorecard_points")
 
             if target_type != "binary":
                 raise ValueError('scaling_method "pd_odds" is not supported '
                                  'for a continuous target.')
 
         elif scaling_method == "min_max":
-            default_keys = ["min", "max"]
+            default_keys = ("min", "max")
 
         if set(scaling_method_params.keys()) != set(default_keys):
             raise ValueError("scaling_method_params must be {} given "
@@ -102,6 +106,17 @@ def _check_scorecard_scaling(scaling_method, scaling_method_params,
                                  .format(scaling_method_params["min"],
                                          scaling_method_params["max"]))
 
+            if rounding:
+                score_min = scaling_method_params['min']
+                if int(score_min) != score_min:
+                    raise ValueError("min must be an integer if rounding=True"
+                                     "; got {}.".format(score_min))
+
+                score_max = scaling_method_params['max']
+                if int(score_max) != score_max:
+                    raise ValueError("max must be an integer if rounding=True"
+                                     "; got {}.".format(score_max))
+
 
 def _compute_scorecard_points(points, binning_tables, method, method_data,
                               intercept, reverse_scorecard):
@@ -377,7 +392,9 @@ class Scorecard(Base, BaseEstimator):
         self._check_is_fitted()
 
         X_t = X[self.binning_process_.variable_names]
-        X_t = self.binning_process_.transform(X_t, metric="indices")
+        X_t = self.binning_process_.transform(
+            X_t, metric="indices", metric_special="empirical",
+            metric_missing="empirical")
 
         score_ = np.zeros(X_t.shape[0])
         selected_variables = self.binning_process_.get_support(names=True)
@@ -439,7 +456,7 @@ class Scorecard(Base, BaseEstimator):
             raise TypeError("path must be a string.")
 
         with open(path, "rb") as f:
-            return dill.load(f)
+            return pickle.load(f)
 
     def save(self, path):
         """Save scorecard to pickle file.
@@ -453,7 +470,7 @@ class Scorecard(Base, BaseEstimator):
             raise TypeError("path must be a string.")
 
         with open(path, "wb") as f:
-            dill.dump(self, f)
+            pickle.dump(self, f)
 
     def _fit(self, X, y, sample_weight, metric_special, metric_missing,
              show_digits, check_input):
@@ -483,6 +500,7 @@ class Scorecard(Base, BaseEstimator):
 
         _check_scorecard_scaling(self.scaling_method,
                                  self.scaling_method_params,
+                                 self.rounding,
                                  self._target_dtype)
 
         # Check sample weight
@@ -553,11 +571,12 @@ class Scorecard(Base, BaseEstimator):
         binning_tables = []
         for i, variable in enumerate(selected_variables):
             optb = self.binning_process_.get_binned_variable(variable)
-            binning_table = optb.binning_table.build(add_totals=False)
+            binning_table = optb.binning_table.build(
+                show_digits=show_digits, add_totals=False)
             c = coefs.ravel()[i]
-            binning_table["Variable"] = variable
-            binning_table["Coefficient"] = c
-            binning_table["Points"] = binning_table[bt_metric] * c
+            binning_table.loc[:, "Variable"] = variable
+            binning_table.loc[:, "Coefficient"] = c
+            binning_table.loc[:, "Points"] = binning_table[bt_metric] * c
 
             nt = len(binning_table)
             if metric_special != 'empirical':
@@ -566,10 +585,10 @@ class Scorecard(Base, BaseEstimator):
                 else:
                     n_specials = 1
 
-                binning_table.loc[nt-1-n_specials:nt-2].at[
-                                 "Points"] = metric_special * c
+                binning_table.loc[
+                    nt-1-n_specials:nt-2, "Points"] = metric_special * c
             elif metric_missing != 'empirical':
-                binning_table.loc[nt-1].at["Points"] = metric_missing * c
+                binning_table.loc[nt-1, "Points"] = metric_missing * c
 
             binning_table.index.names = ['Bin id']
             binning_table.reset_index(level=0, inplace=True)
@@ -585,18 +604,21 @@ class Scorecard(Base, BaseEstimator):
                 points, binning_tables, self.scaling_method,
                 self.scaling_method_params, intercept, self.reverse_scorecard)
 
-            df_scorecard["Points"] = scaled_points
+            df_scorecard.loc[:, "Points"] = scaled_points
 
         if self.intercept_based:
             scaled_points, self.intercept_ = _compute_intercept_based(
                 df_scorecard)
-            df_scorecard["Points"] = scaled_points
+            df_scorecard.loc[:, "Points"] = scaled_points
 
         time_rounding = time.perf_counter()
         if self.rounding:
             points = df_scorecard["Points"]
-            if self.scaling_method == "pdo_odds":
+            if self.scaling_method in ("pdo_odds", None):
                 round_points = np.rint(points)
+
+                if self.intercept_based:
+                    self.intercept_ = np.rint(self.intercept_)
             elif self.scaling_method == "min_max":
                 round_mip = RoundingMIP()
                 round_mip.build_model(df_scorecard)
@@ -609,7 +631,10 @@ class Scorecard(Base, BaseEstimator):
                     # Back-up method
                     round_points = np.rint(points)
 
-            df_scorecard["Points"] = round_points
+                if self.intercept_based:
+                    self.intercept_ = np.rint(self.intercept_)
+
+            df_scorecard.loc[:, "Points"] = round_points
         self._time_rounding = time.perf_counter() - time_rounding
 
         self._df_scorecard = df_scorecard
